from b24pysdk.version import SDK_VERSION


def main():
    """Prints version information for python-telegram-bot, the Bot API and Python."""
    print(f"b24pysdk version {SDK_VERSION}")


if __name__ == "__main__":
    main()
